"use strict";
var __assign = (this && this.__assign) || function () {
    __assign = Object.assign || function(t) {
        for (var s, i = 1, n = arguments.length; i < n; i++) {
            s = arguments[i];
            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
                t[p] = s[p];
        }
        return t;
    };
    return __assign.apply(this, arguments);
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.Plebpack = void 0;
var lodash_flatten_1 = __importDefault(require("lodash.flatten"));
var friendly_errors_webpack_plugin_1 = __importDefault(require("friendly-errors-webpack-plugin"));
var Plebpack = (function () {
    function Plebpack(pkg) {
        this.pkg = pkg;
        this.mode = 'development';
        this.debug = false;
        this.hooks = new Set();
        this.context = process.cwd();
        this.entries = new Map();
        this.aliases = new Map();
        this.plugins = new Set();
        this.loaders = new Set();
        this.extensions = new Set(['*', '.js', '.json']);
        this.resolvePaths = new Set();
        this.externals = new Map();
        this.configs = new Set();
    }
    Plebpack.prototype.getPkg = function () {
        return this.pkg;
    };
    Plebpack.prototype.setDebug = function (debug) {
        this.debug = debug;
    };
    Plebpack.prototype.use = function () {
        var _this = this;
        var hooks = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            hooks[_i] = arguments[_i];
        }
        lodash_flatten_1.default(hooks).forEach(function (hook, position) {
            _this.hooks.add({ run: hook, position: position });
        });
    };
    Plebpack.prototype.addEntry = function (name, path) {
        if (this.entries.has(name)) {
            throw new Error("Duplicate name \"" + name + "\". Entries must be unique.");
        }
        this.entries.set(name, path);
    };
    Plebpack.prototype.getEntries = function () {
        if (this.entries.size === 0) {
            throw new Error('No entry specified.');
        }
        var entries = {};
        this.entries.forEach(function (path, name) {
            entries[name] = path;
        });
        return entries;
    };
    Plebpack.prototype.setOutput = function (output) {
        this.output = output;
    };
    Plebpack.prototype.getOutput = function () {
        if (!this.output) {
            throw new Error('Not output specified.');
        }
        return this.output;
    };
    Plebpack.prototype.setContext = function (context) {
        this.context = context;
    };
    Plebpack.prototype.getContext = function () {
        return this.context;
    };
    Plebpack.prototype.addAlias = function (alias, path) {
        this.aliases.set(alias, path);
    };
    Plebpack.prototype.getAliases = function () {
        var aliases = {};
        this.aliases.forEach(function (path, name) {
            aliases[name] = path;
        });
        return aliases;
    };
    Plebpack.prototype.addPlugin = function (plugin, priority) {
        if (priority === void 0) { priority = 0; }
        this.plugins.add({ plugin: plugin, priority: priority });
    };
    Plebpack.prototype.getPlugins = function () {
        var plugins = [];
        this.plugins.forEach(function (plugin) {
            plugins.push(plugin);
        });
        return plugins
            .map(function (plugin, position) { return (__assign(__assign({}, plugin), { position: position })); })
            .sort(function (a, b) {
            if (a.priority === b.priority) {
                return (a.position = b.position);
            }
            return b.priority - a.priority;
        })
            .map(function (plugin) { return plugin.plugin; });
    };
    Plebpack.prototype.addLoader = function (loader) {
        this.loaders.add(loader);
    };
    Plebpack.prototype.getLoaders = function () {
        var loaders = [];
        this.loaders.forEach(function (loader) {
            loaders.push(loader);
        });
        return loaders;
    };
    Plebpack.prototype.addExtension = function (extension) {
        this.extensions.add(extension);
    };
    Plebpack.prototype.getExtensions = function () {
        var extensions = [];
        this.extensions.forEach(function (extension) {
            extensions.push(extension);
        });
        return extensions.length > 0 ? extensions : undefined;
    };
    Plebpack.prototype.addResolvePath = function (path) {
        this.resolvePaths.add(path);
    };
    Plebpack.prototype.getResolvePaths = function () {
        var resolvePaths = [];
        this.resolvePaths.forEach(function (path) {
            resolvePaths.push(path);
        });
        return resolvePaths.length > 0 ? resolvePaths : undefined;
    };
    Plebpack.prototype.addExternal = function (name, path) {
        if (this.externals.has(name)) {
            throw new Error("Duplicate name \"" + name + "\". Externals must be unique.");
        }
        this.externals.set(name, path);
    };
    Plebpack.prototype.getExternals = function () {
        var externals = {};
        this.externals.forEach(function (path, name) {
            externals[name] = path;
        });
        return externals;
    };
    Plebpack.prototype.merge = function (config) {
        if (typeof config === 'function') {
            this.configs.add(config);
        }
        this.configs.add(function (c) { return (__assign(__assign({}, c), config)); });
    };
    Plebpack.prototype.getConfigs = function () {
        return this.configs;
    };
    Plebpack.prototype.config = function (mode) {
        var _this = this;
        var hooks = [];
        this.mode = mode;
        this.hooks.forEach(function (hook) {
            hooks.push(hook);
        });
        hooks
            .sort(function (a, b) {
            return a.position - b.position;
        })
            .forEach(function (hook) { return hook.run(_this); });
        this.addPlugin(new friendly_errors_webpack_plugin_1.default());
        var config = {
            mode: this.mode,
            context: this.getContext(),
            devtool: 'source-map',
            entry: this.getEntries(),
            output: this.getOutput(),
            module: {
                rules: this.getLoaders(),
            },
            plugins: this.getPlugins(),
            performance: {
                hints: false,
            },
            resolve: {
                extensions: this.getExtensions(),
                alias: this.getAliases(),
                modules: this.getResolvePaths(),
            },
            externals: this.getExternals(),
        };
        if (this.mode === 'development') {
            config.devtool = 'inline-source-map';
        }
        this.getConfigs().forEach(function (callback) {
            config = callback(config);
        });
        if (this.debug) {
            console.log(JSON.stringify(config, null, 2));
        }
        return config;
    };
    return Plebpack;
}());
exports.Plebpack = Plebpack;
//# sourceMappingURL=plebpack.js.map